/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: gale_transform.cc,v 1.17 2004/06/22 16:23:09 gawrilow Exp $"

#include <Rational.h>
#include <Poly.h>
#include <Matrix.h>
#include <linalg.h>

namespace polymake { namespace polytope {

void gale_transform(Poly& p)
{
   const Matrix<Rational> V=p.give("VERTICES");
   const Matrix<Rational> G=T(null_space(T(V)));
   p.take("GALE_TRANSFORM") << G;
}

} }

using namespace polymake;

/** @file gale_transform
 *
 *  Calculate the Gale transform matrix for vertices of a given polytope.
 *  Polytope must be bounded.
 *
 *  @synopsis gale_transform <file>
 *
 *  @reading  BOUNDED, VERTICES
 *  @writing  GALE_TRANSFORM
 */

int main(int argc, const char *argv[]) {
   if (argc != 2 ) {
      cerr << "usage: " << argv[0] << " <file>" << endl;
      return 1;
   }
   try {
      Poly p(argv[1], ios::in | ios::out);
      polytope::gale_transform(p);
   }
   catch (const std::exception& e) {
      cerr << e.what() << endl;
      return 1;
   }
   return 0;
}
