/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: ball.cc,v 1.5 2005/05/12 11:05:10 witte Exp $"

#include <Poly.h>
#include <Matrix.h>
#include <Array.h>
#include <Set.h>

namespace polymake {
namespace topaz {
  
void ball (Poly& p, const int dim)
{
   const int g_dim= dim==0 ? 1 : dim;
   Matrix<int> Geom(dim+1,g_dim);
   for (int i=0; i<dim; ++i)
      Geom(i+1,i) = 1;
      
   p.take("DESCRIPTION") << "A " << dim
			 << "-dimensional topological ball.\nRealized as a "
			 << dim << "-simplex.";
   p.take("FACETS") << sequence(0, dim+1);
   p.take("DIM") << dim;
   p.take("PURE") << 1;
   p.take("MANIFOLD") << 1;
   p.take("BALL")<< 1;
   p.take("ORIENTED_PSEUDO_MANIFOLD") << 1;
   p.take("GEOMETRIC_REALIZATION") << Geom;
}
   
} }

using namespace polymake;

/** @file ball
 *
 *  Produce a $d$-ball.
 *
 *  @synopsis ball <file> <dimension>
 *
 *  @writing  DESCRIPTION, FACETS, DIM, PURE, MANIFOLD, ORIENTED_PSEUDO_MANIFOLD, BALL, GEOMETRIC_REALIZATION DESCRIPTION
 *
 *  @index clients Producing from scratch
 */

int main(int argc, const char *argv[])
{
   if (argc!=3) {
      cerr << "usage: " << argv[0] << " <file> <dimension>" << endl;
      return 1;
   }
   try {
      Poly p(argv[1], ios::out | ios::trunc, "topaz::default");
      
      const int dim=atoi(argv[2]);
      if (dim<0) {
	 cerr << argv[0] << ": dim<0" << endl;
	 return 1;
      }
      
      topaz::ball(p, dim);
   }
   catch (const std::exception& e) {
      cerr << e.what() << endl;
      return 1;
   }
   return 0;
}


// Local Variables:
// mode:C++
// c-basic-offset:3
// End:
