/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: is_ball.cc,v 1.1 2005/09/23 14:43:51 witte Exp $"

#include <Poly.h>
#include <Array.h>
#include <complex_tools.h>

#include <string>

namespace polymake {
namespace topaz {
  
namespace {

enum { quiet };

}

void is_ball(Poly& p)
{
   const Array< Set<int> > C = p.give("FACETS");
   const int d = p.give("DIM");
   const bool is_cpmf = p.give("CLOSED_PSEUDO_MANIFOLD");

   
   bool is_bos = true;
   if (d>2)
      throw std::runtime_error("is_ball: Dimension of the complex must be smaller 4.");
   
   else if (d==1)
      is_bos = C1_is_BOS(C);
   
   else if (d==2)
      is_bos = C2_is_BOS(C);
   
   p.take("BALL") << (is_bos && !is_cpmf);
}
   
} }

using namespace polymake;

/** @file is_ball
 *
 *  Check whether a complex is a ball.
 *  The client works for balls of dim 2 or smaller only.
 *
 *  @synopsis is_ball <file>
 *
 *  @reading  FACETS, DIM, CLOSED_PSEUDO_MANIFOLD
 *  @writing  BALL
 */

int main(int argc, const char* argv[])
{
   if (argc!=2) {
      cerr << "usage: " << argv[0] << " <file>" << endl;
      return 1;
   }
   try {
      Poly p(argv[1], ios::in | ios::out);
      topaz::is_ball(p);
   }
   catch (const std::exception& e) {
      cerr << e.what() << endl;
      return 1;
   }
   return 0;
}


// Local Variables:
// mode:C++
// c-basic-offset:3
// End:
