/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: is_closed_pseudo_manifold.cc,v 1.9 2004/10/26 18:57:01 gawrilow Exp $"

#include <Poly.h>
#include <Array.h>
#include <complex_tools.h>

#include <string>

namespace polymake {
namespace topaz {
  
void is_closed_pseudo_manifold(Poly& p)
{
      const HasseDiagram<> HD = p.give("HASSE_DIAGRAM");

      p.take("CLOSED_PSEUDO_MANIFOLD") << is_closed_pseudo_manifold(HD);
}

} }

using namespace polymake;

/** @file is_closed_pseudo_manifold
 *
 *  Check whether a complex is a closed pseudo-manifold.
 *
 *  @synopsis is_closed_pseudo_manifold <file>
 *
 *  @reading  HASSE_DIAGRAM
 *  @writing  CLOSED_PSEUDO_MANIFOLD
 */


int main(int argc, const char* argv[])
{
   if (argc!=2) {
      cerr << "usage: " << argv[0] << " <file>" << endl;
      return 1;
   }
   try {
      Poly p(argv[1], ios::in | ios::out);
      topaz::is_closed_pseudo_manifold(p);
   }
   catch (const std::exception& e) {
      cerr << e.what() << endl;
      return 1;
   }
   return 0;
}


// Local Variables:
// mode:C++
// c-basic-offset:3
// End:
