/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: odd_complex_of_manifold.cc,v 1.9 2004/10/26 18:57:01 gawrilow Exp $"

#include <Poly.h>
#include <complex_tools.h>

#include <ext/hash_set>

namespace polymake {
namespace topaz {

void odd_complex_of_manifold(Poly& p)
{
   const Array< Set<int> > C = p.give("FACETS");
   const bool is_mf = p.give("MANIFOLD");
   if (!is_mf)
      throw std::runtime_error("odd_complex_of_manifold: Complex is not a MANIFOLD");   
   HasseDiagram<> HD;
   if (!(p.lookup("HASSE_DIAGRAM") >> HD))
      HD = hasse_diagram(C,C[0].size()-1,-2);
   
   if (C[0].size()-1 < 2)
      throw std::runtime_error("odd_complex_of_manifold: DIM of complex must br greater 2.");
   
   // create hash set for the facets of the Boundary
   const Array< Set<int> > Bound = p.give("BOUNDARY_OF_PSEUDO_MANIFOLD");
   const PowerSet<int> Bound_sk = Bound[0].empty() ? PowerSet<int>() :
      k_skeleton(Bound, Bound[0].size()-2);
   
   std_ext::hash_set< Set<int> > Boundary(Bound_sk.size());
   for (Entire< PowerSet<int> >::const_iterator c_it=entire(Bound_sk); !c_it.at_end(); ++c_it)
      Boundary.insert(*c_it);
   
   bool output = false;
   std::list< Set<int> > odd_complex;
   for (Entire<sequence>::iterator f=entire(HD.nodes_of_dim(-3)); !f.at_end(); ++f)
      if ( HD.out_edges(*f).size() % 2 && Boundary.find(HD.node(*f)) ==  Boundary.end()) {
	 output = true;
	 odd_complex.push_back(HD.node(*f));
      }
   
   if (output)
      p.take("ODD_SUBCOMPLEX") << as_array(odd_complex);
   else
      p.take("ODD_SUBCOMPLEX") << Set<int>();
}
   
} }

using namespace polymake;

/** @file odd_complex_of_manifold
 *
 *  Find all faces of co-dimension -2 of which have a non bipartied link.
 *
 *  @synopsis odd_complex_of_manifold <file>
 */


int main(int argc, const char* argv[])
{
   if (argc != 2) {
      cerr << "usage: " << argv[0] << " <file>" << endl;
      return 1;
   }
   try {
      Poly p(argv[1], ios::in | ios::out);
      topaz::odd_complex_of_manifold(p);
   }
   catch (const std::exception& e) {
      cerr << e.what() << endl;
      return 1;
   }
   return 0;
}


// Local Variables:
// mode:C++
// c-basic-offset:3
// End:
