/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: projective_plane.cc,v 1.7 2004/10/26 18:57:01 gawrilow Exp $"

#include <Poly.h>
#include <complex_tools.h>

namespace polymake {
namespace topaz {
  
void projective_plane(Poly& p)
{
   p.take("DESCRIPTION") << "The projective plane.";
   p.take("FACETS") << projective_plane();
   p.take("DIM") << 2;
   p.take("MANIFOLD") << 1;
   p.take("CLOSED_PSEUDO_MANIFOLD") << 1;
   p.take("ORIENTED_PSEUDO_MANIFOLD") << 0;
}

} }

using namespace polymake;

/** @file projective_plane
 *
 *  The projective plane.
 *
 *  @synopsis projective_plane <file>
 *
 *  @writing  DESCRIPTION, FACETS, DIM, MANIFOLD, CLOSED_PSEUDO_MANIFOLD, ORIENTED_PSEUDO_MANIFOLD
 *
 *  @index clients Producing from scratch
 */

int main(int argc, const char *argv[])
{
   if (argc!=2) {
      cerr << "usage: " << argv[0] << " <file>" << endl;
      return 1;
   }
   try {
      Poly p(argv[1], ios::out | ios::trunc, "topaz::default");
      topaz::projective_plane(p);
   }
   catch (const std::exception& e) {
     cerr << e.what() << endl;
     return 1;
   }
   return 0;
}


// Local Variables:
// mode:C++
// c-basic-offset:3
// End:
