/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: t_dual_graph.cc,v 1.8 2004/10/26 18:57:00 gawrilow Exp $"

#include <Poly.h>
#include <t_graph.h>

namespace polymake {
namespace topaz {
  
void t_dual_graph(Poly& p)
{
     const FacetList C = p.give("FACETS");

     p.take("DUAL_GRAPH") << t_dual_graph(C);
}

} }

using namespace polymake;

/** @file t_dual_graph
 *
 *  Produce the dual graph of a simplicial complex.
 *
 *  @synopsis t_dual_graph <file>
 *
 *  @reading  FACETS
 *  @writing  DUAL_GRAPH
 */

int main(int argc, const char* argv[])
{
   if (argc!=2) {
      cerr << "usage: " << argv[0] << " <file>" << endl;
      return 1;
   }
   try {
      Poly p(argv[1], ios::in | ios::out);
      topaz::t_dual_graph(p);
   } 
   catch (const std::exception& e) {
     cerr << e.what() << endl;
     return 1;
   }
   return 0;
}


// Local Variables:
// mode:C++
// c-basic-offset:3
// End:
