/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: rational_integer.cc,v 1.13 2004/06/22 16:22:38 gawrilow Exp $"

#include <Poly.h>
#include <Rational.h>
#include <Matrix.h>

namespace polymake { namespace polytope {

void Rational2Integer(Poly& p, const char *in_section, const char *out_section)
{
   Matrix<Rational> X=p.give(in_section);
   for (Entire< Rows< Matrix<Rational> > >::iterator r=entire(rows(X)); !r.at_end(); ++r) {
      Entire< Matrix<Rational>::row_type >::iterator x=entire(*r);
      Integer LCM=denominator(*x);
      for (++x; !x.at_end(); ++x)
	 LCM=lcm(LCM, denominator(*x));
      *r *= LCM;
   }
   p.take(out_section) << X;
}

} }

using namespace polymake;

int main(int argc, const char *argv[]) {
   if (argc < 3 || argc > 4) {
      cerr << "usage: " << argv[0] << " <file> <section> [<out-section>]" << endl;
      return 1;
   }
   try {
      Poly p(argv[1], ios::in | ios::out);
      polytope::Rational2Integer(p, argv[2], argv[argc-1]);
   }
   catch (const std::exception& e) {
      cerr << e.what() << endl;
      return 1;
   }
   return 0;
}
