/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: max_cliques.cc,v 1.3 2004/06/22 16:22:36 gawrilow Exp $"

#include <Poly.h>
#include <max_cliques.h>

namespace polymake { namespace graph {

void max_cliques(Poly& p, const char *graph_section, const char *result_section)
{
   const Graph<> G=p.give(graph_section);
   const PowerSet<int> maxCliques = max_cliques(G);
   p.take(result_section) << as_array(maxCliques);
}

} }

using namespace polymake;

/** @file max_cliques
 *
 *  Computes all inclusion maximal cliques of a graph.
 *
 *  @synopsis max_cliques <file> <max_cliques_section> <graph_section>
 *
 *  @reading  <graph_section>
 *  @writing  <max_cliques_section>
 */

int main(int argc, const char *argv[])
{
   if (argc!=4) {
      cerr << "usage: " << argv[0] << " <file> <graph_section> <max_cliques_section>\n";
      return 1;
   }
   try {
      Poly p(argv[1], ios::in | ios::out);
      graph::max_cliques(p, argv[2], argv[3]);
   }
   catch (const std::exception& e) {
      cerr << e.what() << endl;
      return 1;
   }
   return 0;
}

// Local Variables:
// c-basic-offset:3
// End:
